#!/usr/bin/env python

import argparse
import json
import doxscrapers.scrapers
import config

SITES = {
    "facebook": doxscrapers.scrapers.facebook,
    "instagram": doxscrapers.scrapers.instagram,
    "youtube": doxscrapers.scrapers.youtube,
    "twitter": doxscrapers.scrapers.twitter,
    "twitch": doxscrapers.scrapers.twitch,
    "gplus": doxscrapers.scrapers.gplus,
}

PARSER = argparse.ArgumentParser(description="Scrapes information about a "
                                             "user's social networking account")

PARSER.add_argument('-u', '--user', required=True,
                    help="The name of a social networking account to measure")
PARSER.add_argument('-s', '--site', choices=SITES.keys(), required=True,
                    help="The social networking service to measure")
PARSER.add_argument('-b', '--binary', default=None,
                    help="Path to the firefox binary (Default: result of "
                         "`which`)")
PARSER.add_argument('-l', '--logfile', default=None,
                    help="File to log Selenium firefox output to.")

ARGS = PARSER.parse_args()

SCRAPER = SITES[ARGS.site]
if ARGS.site == "twitter":
    SCRAPER_OUTPUT = SCRAPER(ARGS.user, config.twitter_auth_params, binary_path=ARGS.binary, log_file=ARGS.logfile)
else:
    SCRAPER_OUTPUT = SCRAPER(ARGS.user, binary_path=ARGS.binary, log_file=ARGS.logfile)
print json.dumps(SCRAPER_OUTPUT)
