#!/usr/bin/env python

import sys
import argparse
import json
import doxcommentextractor.comments
from doxcommentextractor.types import CommentLabel
from doxcommentextractor.abuse import is_abusive_text, CLASSIFICATIONS

PARSER = argparse.ArgumentParser(description="Extract comments from HTML")
PARSER.add_argument('-n', '--network', required=True,
                    choices=["facebook", "instagram"],
                    help="The OSN the HTML came from")
PARSER.add_argument('-o', '--output', choices=("text", "json"), default="json",
                    help="The format to print the extracted comments in. "
                         "Either 'text', meaning easily human-readable output, "
                         "or 'json'")
ARGS = PARSER.parse_args()

HTML = sys.stdin.read()
METHOD = None
if ARGS.network == "facebook":
    METHOD = doxcommentextractor.comments.facebook
else:
    METHOD = doxcommentextractor.comments.instagram

IS_SUCCESS, COMMENTS = METHOD(HTML)
if not IS_SUCCESS:
    print "Error parsing HTML:"
    print COMMENTS
    sys.exit(1)

def comment_to_comment_label(a_comment):
    is_text_abusive = is_abusive_text(a_comment.text) == CLASSIFICATIONS.is_abuse
    return CommentLabel(a_comment, is_text_abusive)

# Ignored if we're outputting a JSON record.
if ARGS.output == "json":
    JSON_ITEMS = []
else:
    print "Found {} comments".format(len(COMMENTS))

for A_LABELED_COMMENT in [comment_to_comment_label(C) for C in COMMENTS]:
    A_COMMENT = A_LABELED_COMMENT.comment
    IS_ABUSE = A_LABELED_COMMENT.is_abusive
    if ARGS.output == "text":
        print u" - {}: {} ({}_".format(
            A_COMMENT.commenter,
            A_COMMENT.text,
            "abuse" if IS_ABUSE else "not abuse"
        )
    else:
        JSON_ITEMS.append({
            "user": A_COMMENT.commenter,
            "comment": A_COMMENT.text,
            "abuse": IS_ABUSE
        })

if ARGS.output == "json":
    print json.dumps(JSON_ITEMS)
