import logging
import pprint
import requests


LOGGER = logging.getLogger("dox-account-recrawler.log")


def _post_request(url, params):
    try:
        response = requests.post(url, data=params).json()
    except (requests.exceptions.HTTPError, requests.exceptions.Timeout,
            requests.exceptions.ConnectionError), exc:
        log_params = {
            u"url": url,
            u"error": str(exc)
        }
        LOGGER.error(u"HTTP error requesting url: %(url)s, received: %(error)s",
                     log_params)
        return None
    except ValueError, exc:
        log_params = {
            u"url": url,
            u"error": str(exc)
        }
        LOGGER.error(u"Error parsing JSON from url: %(url)s, received: %(error)s",
                     log_params)
        return None

    if 'error' not in response:
        log_params = {
            u"url": url,
            u"response": pprint.pformat(response)
        }
        LOGGER.error(u"Unexpected format of response for '%(url)s' "
                     u"request: %(response)s", log_params)
        return None

    if response['error']:
        log_params = {
            u"url": url,
            u"error": response['msg'],
        }
        LOGGER.error(u"Received application error when requesting %(url)s: "
                     u"%(error)s", log_params)
        return None

    return response


def accounts_to_recrawl_for_num(db_connector_host, recrawl_num):

    to_recrawl_url = "http://{}/accounts-to-recrawl".format(db_connector_host)
    response = _post_request(to_recrawl_url, {"num": recrawl_num})
    if response is None:
        return None

    log_params = {
        u"count": len(response['recrawls']),
        u"recrawl_num": recrawl_num,
    }
    LOGGER.info(u"Successfully recieved %(count)s accounts to recrawl for "
                u"their %(recrawl_num)s crawl", log_params)
    return response['recrawls']


def recrawl_account(scraper_host, account, service):

    social_scraper_url = "http://{}/scrape".format(scraper_host)
    post_params = {
        "account": account,
        "service": service,
    }
    response = _post_request(social_scraper_url, post_params)
    if response is None:
        return None

    log_params = post_params
    LOGGER.info("Successfully received new crawl for '%(account)s' on '%(service)s'",
                log_params)

    return response['msg']


def submit_recrawl(db_connector_host, recrawl_text, recrawl_account_id):

    record_recrawl_url = "http://{}/record-crawl".format(db_connector_host)
    post_params = {
        "account_id": recrawl_account_id,
        "crawl": recrawl_text,
    }
    response = _post_request(record_recrawl_url, post_params)
    if response is None:
        return None

    log_params = {
        u"account_id": recrawl_account_id,
        u"response": response['msg'],
    }
    LOGGER.info(u"Successfully recorded recrawl for '%(account_id)s', received: "
                u"%(response)s", log_params)
    return response


def recrawl_all(db_connector_host, scraper_host, num):
    to_recrawl = accounts_to_recrawl_for_num(db_connector_host, num)
    if to_recrawl is None:
        return None

    successes = 0
    for item in to_recrawl:
        account_id = item['account_id']
        account_name = item['account_name']
        service_name = item['service_name']

        crawl_text = recrawl_account(scraper_host, account_name, service_name)
        if crawl_text is None:
            return None

        submit_rs = submit_recrawl(db_connector_host, crawl_text, account_id)
        if submit_rs is None:
            return None
        successes += 1
    return successes
