import os
import logging
from logging.handlers import TimedRotatingFileHandler


def http_error_logger(request_handler):
    status = request_handler.get_status()
    http_request = request_handler.request
    params = dict(
        remote_ip=http_request.remote_ip,
        status=status,
        uri=http_request.uri,
    )

    logger = logging.getLogger("tornado.access")

    if status < 400:
        logger.info("%(remote_ip)s %(status)d %(uri)s", params)
    else:
        logger.error("%(remote_ip)s %(status)d %(uri)s", params)


def configure_logger(log_dir, debug):

    format_str = "%(asctime)s"

    if not log_dir:
        return

    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    access_log_file = os.path.join(log_dir, "dox-classifier.access.log")
    access_log_handler = TimedRotatingFileHandler(access_log_file,
                                                  when="midnight")
    access_log_formatter = logging.Formatter("%(created)f|%(message)s")
    access_log_handler.setFormatter(access_log_formatter)
    access_logger = logging.getLogger('tornado.access')
    access_logger.addHandler(access_log_handler)

    error_log_file = os.path.join(log_dir, "dox-classifier.error.log")
    error_log_handler = TimedRotatingFileHandler(error_log_file,
                                                 when="midnight")
    error_log_formatter = logging.Formatter(format_str)
    error_log_handler.setFormatter(error_log_formatter)
    error_logger = logging.getLogger('tornado.application')
    error_logger.addHandler(error_log_handler)

    gen_log_file = os.path.join(log_dir, "dox-classifier.gen.log")
    gen_log_handler = TimedRotatingFileHandler(gen_log_file,
                                               when="midnight")
    gen_log_formatter = logging.Formatter(format_str)
    gen_log_handler.setFormatter(gen_log_formatter)
    gen_logger = logging.getLogger('tornado.general')
    gen_logger.addHandler(gen_log_handler)
    gen_logger.info("dox-classifier server started.")

    for a_logger in (access_logger, error_logger, gen_logger):
        a_logger.setLevel(logging.DEBUG if debug else logging.WARNING)
        a_logger.propagate = False
