import urllib
import json
import uuid
import pprint
from HTMLParser import HTMLParser
import tornado.httpclient
import tornado.web
import tornado.gen
from tornado.log import gen_log
import html2text

SOURCES_TO_STRIP_HTML = ("fourchan-pol", "fourchan-b", "eightchan-pol",
                         "eightchan-baphomet")

TOKENS = {
    u'ONLINE:ig': "instagram",
    u'ONLINE:twitter': "twitter",
    u'ONLINE:yt': "youtube",
    u'ONLINE:gplus': "gplus",
    u'ONLINE:fb': "facebook",
    u'ONLINE:twitch': "twitch",
}

URLS = {}
def set_urls(new_urls):
    for url_name, url_value in new_urls.items():
        URLS[url_name] = url_value


class ReceiveTextHandler(tornado.web.RequestHandler):

    def __init__(self, *args, **kwargs):
        self.text = None
        self.paste_id = None
        self.request_id = None
        self.is_dox = None
        self.source = None
        self.found_services = None
        self.service_index = 0
        self.account_id = None
        super(ReceiveTextHandler, self).__init__(*args, **kwargs)


    def _validate_response(self, response, expected_params, service_name):

        error = None
        if response.error is not None:
            error = ("Received error from '{}' service\n"
                     "{}".format(service_name, str(response.error)))
            self._send_error({"error": True, "msg": error})
            return True, None

        gen_log.debug("%(request_id)s: Validing response from %(url)s, status "
                      "%(code)s, body %(body)s", {
                          "request_id": self.request_id,
                          "code": response.code,
                          "url": response.request.url,
                          "body": response.body,
                      })

        try:
            received_text = response.body
            received_record = json.loads(received_text)
            found_params = {}
            found_params['error'] = received_record['error']
            found_params['msg'] = received_record['msg']
            if found_params['error']:
                self._send_error({"error": True, "msg": found_params['msg']})
                return True, None

            for expected_key in expected_params:
                found_params[expected_key] = received_record[expected_key]

        except ValueError, exc:
            error = ("Received invalid JSON from the '{}' service.\n"
                     "Recieved: {}".format(service_name, received_text))
        except KeyError, exc:
            error = ("Received unexpectedly formatted JSON from the '{}' service.\n"
                     "Missing key: {}".format(service_name, exc.message))
        if error:
            self._send_reply({"error": True, "msg": error})
            return True, None
        return False, found_params


    def _send_post(self, url, params, callback):
        encoded_dict = {}
        for post_key, post_value in params.iteritems():
            if isinstance(post_value, unicode):
                post_value = post_value.encode('utf8')
            elif isinstance(post_value, str):
                post_value.decode('utf8')
            encoded_dict[post_key] = post_value

        post_body = urllib.urlencode(encoded_dict)
        http_client = tornado.httpclient.AsyncHTTPClient()
        gen_log.debug("%(request_id)s: Posting to %(url)s, contents %(body)s", {
            "request_id": self.request_id,
            "url": url,
            "body": post_body
        })
        http_client.fetch(url, method="POST", body=post_body, callback=callback)


    def _send_error(self, reply):
        gen_log.error("%(request_id)s: %(msg)s", {
            "request_id": self.request_id,
            "msg": reply['msg'],
        })
        self.write(reply)
        self.finish()


    def _send_reply(self, reply):
        gen_log.debug("%(request_id)s: Replying with %(msg)s", {
            "request_id": self.request_id,
            "msg": pprint.pformat(reply),
        })
        self.write(reply)
        self.finish()


    @tornado.web.asynchronous
    def post(self):
        self.request_id = str(uuid.uuid4())
        gen_log.debug("%(request_id)s: Received request", {
            "request_id": self.request_id
        })

        try:
            json_text = self.get_body_argument("text")
            source = self.get_body_argument('source')
        except tornado.web.MissingArgumentError, exc:
            reply = {
                "msg": "Missing POST argument '{}'".format(exc.arg_name),
                "error": True,
            }
            self._send_error(reply)
            return

        self.text = json_text
        self.source = source

        was_paste_recorded_params = {u"text": json_text, u"source": source}
        self._send_post(URLS['HAS_PASTE_BEEN_RECORDED_URL'],
                        was_paste_recorded_params,
                        self.on_was_paste_recorded_response)


    def on_was_paste_recorded_response(self, response):
        reply = {'error': True}
        service_name = 'has paste been recorded'
        error, received_params = self._validate_response(response, ['paste_id'],
                                                         service_name)
        if error:
            return

        if received_params['paste_id'] is not None:
            paste_id = received_params['paste_id']
            reply['msg'] = "Text previously seen with id {}".format(paste_id)
            reply['paste_id'] = received_params['paste_id']
            reply['error'] = False
            self._send_reply(reply)
            return

        paste_text = self._stripped_paste_text()
        is_dox_params = {u"text": paste_text}
        self._send_post(URLS['CLASSIFY_TEXT_URL'], is_dox_params,
                        self.on_is_text_dox_response)


    def on_is_text_dox_response(self, response):
        service_name = 'is text a dox'
        error, received_params = self._validate_response(response, [], service_name)
        if error:
            return

        self.is_dox = True if received_params['msg'] == "dox" else False

        record_text_params = {
            u"text": self.text,
            u"source": self.source,
            u"is_dox": self.is_dox
        }
        self._send_post(URLS['RECORD_PASTE_URL'], record_text_params,
                        self.on_recorded_text_response)


    def on_recorded_text_response(self, response):
        reply = {'error': True}
        service_name = "record text"
        error, received_params = self._validate_response(response, ['paste_id'],
                                                         service_name)
        if error:
            return

        self.paste_id = received_params['paste_id']

        if not self.is_dox:
            reply['msg'] = "Submitted text is not a dox"
            reply["paste_id"] = self.paste_id
            reply['error'] = False
            self._send_reply(reply)
            return

        paste_text = self._stripped_paste_text()
        extract_tokens_params = {"text": paste_text}
        self._send_post(URLS['EXTRACT_TOKENS_URL'], extract_tokens_params,
                        self.on_extract_tokens)


    def on_extract_tokens(self, response):
        reply = {"error": True}
        service_name = "extract tokens"
        error, received_params = self._validate_response(response, [], service_name)
        if error:
            return

        extracted_tokens = received_params['msg']

        found_services = []
        for token_name, social_service in TOKENS.items():
            if token_name not in extracted_tokens:
                continue
            if len(extracted_tokens[token_name]) == 0:
                continue
            found_services.append((social_service, extracted_tokens[token_name][0]))

        if len(found_services) == 0:
            reply = {
                "error": False,
                "is_dox": True,
                "msg": "No social services found in dox file",
                "paste_id": self.paste_id
            }
            self._send_reply(reply)
            return

        self.found_services = found_services
        self.service_index = 0
        self.reduce_social_accounts(None)


    def reduce_social_accounts(self, response):

        found_services = self.found_services
        index = self.service_index

        if index == len(found_services):
            all_service_names = [s[0] for s in found_services]
            reply = {
                "error": None,
                "paste_id": self.paste_id,
                "is_dox": self.is_dox,
                "msg": "scraped accounts for {}".format(", ".join(all_service_names)),
            }
            self._send_reply(reply)
            return

        service_name, account_name = found_services[index]
        post_params = {
            "paste_id": self.paste_id,
            "service": service_name,
            "account": account_name
        }
        self._send_post(URLS['RECORD_SOCIAL_ACCOUNT_URL'], post_params,
                        self.on_record_social_account)


    def on_record_social_account(self, response):
        service_name = "record social account"
        error, received_params = self._validate_response(response, ['account_id'],
                                                         service_name)
        if error:
            return

        found_services = self.found_services
        index = self.service_index
        self.account_id = received_params['account_id']

        service_name, account_name = found_services[index]
        post_params = {u"service": service_name, u"account": account_name}
        self._send_post(URLS['SOCIAL_SCRAPE_URL'], post_params,
                        self.on_scraped_social_account)


    def on_scraped_social_account(self, response):
        service_name = "scrape social service"
        error, received_params = self._validate_response(response, [], service_name)
        if error:
            return

        post_params = {
            u"account_id": self.account_id,
            u"crawl": received_params['msg']
        }
        self.service_index += 1
        self._send_post(URLS['RECORD_SOCIAL_CRAWL_URL'], post_params,
                        self.reduce_social_accounts)

    def _stripped_paste_text(self):
        # The "chan" sites allow users to post HTML, which adds non-useful
        # complexity to the classifier.  For these sites, we strip the
        # HTML from the pastes (and generate a rough text version of the same
        # using the html2text package).
        if self.source not in SOURCES_TO_STRIP_HTML:
            return self.text

        html_parser = HTMLParser()
        submitted_record = json.loads(self.text)
        text_from_html = html2text.html2text(submitted_record['text'], bodywidth=0)
        submitted_record['text'] = html_parser.unescape(text_from_html)
        return json.dumps(submitted_record)
