import re
import doxscrapers.sites.base
from doxscrapers.types import PostTeaser, Post


class SkidPasteDoxScraper(doxscrapers.sites.base.BaseDoxScraper):

    def __init__(self, marshaller_host):
        super(SkidPasteDoxScraper, self).__init__(1, marshaller_host, "skidpaste")

    def items_in_index(self, proxy=None):
        """
        Fetches a list of PostTeaser objects, each representing a posting
        on PasteBin.

        Keyword Args:
            proxy -- An optional URL of a HTTP proxy, that should be used when
                     requesting against the API.

        Returns:
            None on an error, or a list of zero or more PostTeaser objects.
        """
        index_url = "https://skidpaste.org/pastes.html"
        response_dom = self._dom_for_url(index_url, proxy=proxy)
        if response_dom is None:
            return None

        links = response_dom.select("#content_left .table.sortable tr > td:first-child a")
        teasers = [a.get("href") for a in links]

        log_params = {
            u"count": len(teasers),
            u"url": index_url,
        }
        self._logger.info(u"Successfully received %(count)s items from %(url)s",
                          log_params)
        teasers = [PostTeaser(a.get("href")) for a in links]
        return teasers

    def details_for_item(self, item, proxy=None):
        """
        Returns a Post object, describing the full contents of the
        post that was listed in an index of posts.

        Args:
            item -- a doxscrapers.types.PostTeaser object

        Keyword Args:
            proxy -- An optional URL of a HTTP proxy, that should be used when
                     requesting against the API.

        Returns:
            None on an error, and otherwise a Post
            object.
        """
        full_url = "https://skidpaste.org/{}".format(item.key)
        scrape_url = "https://skidpaste.org/{}.text".format(item.key)

        document = self._dom_for_url(full_url, proxy=proxy)
        if document is None:
            return None

        paste_text = self._request_url(scrape_url, proxy=proxy)
        if paste_text is None:
            return None

        title_elm_text = document.select("#content_left .content_paste")[0].text
        matches = re.search(r".*Text \|(.*)\w?-", title_elm_text, re.MULTILINE | re.DOTALL)
        if not matches:
            log_params = {
                "url": full_url,
            }
            self._logger.error("Unable to determine the title of the post at "
                               "%(url)s", log_params)
            return None
        post_title = matches.group(1)

        post = Post(item.key, paste_text, full_url)
        post.title = post_title
        return post
