#!/usr/bin/env python
"""
Command line tool to fetch the posts on a single supported text pasting site.
"""

import argparse
import json
import doxscrapers.reporting
import doxscrapers.sites.pastebin
import doxscrapers.sites.cloudnine
import doxscrapers.sites.kdepaste
import doxscrapers.sites.doxbin
import doxscrapers.sites.chan
import doxscrapers.sites.skidpaste

SITES = {
    "pastebin": doxscrapers.sites.pastebin.PasteBinDoxScraper,
    "cloudnine": doxscrapers.sites.cloudnine.CloudNineDoxScraper,
    "kdepaste": doxscrapers.sites.kdepaste.KdePasteDoxScraper,
    "doxbin": doxscrapers.sites.doxbin.DoxBinDoxScraper,
    "fourchan-pol": doxscrapers.sites.chan.FourChanPolDoxScraper,
    "fourchan-b": doxscrapers.sites.chan.FourChanBDoxScraper,
    "eightchan-pol": doxscrapers.sites.chan.EightChanPolDoxScraper,
    "eightchan-baphomet": doxscrapers.sites.chan.EightChanBaphometDoxScraper,
    "skidpaste": doxscrapers.sites.skidpaste.SkidPasteDoxScraper,
}

PARSER = argparse.ArgumentParser(description="Scrape all posts on a post site "
                                             "and print the results out "
                                             "as JSON.")
PARSER.add_argument('-s', '--site', required=True, choices=SITES.keys(),
                    help="The dox hosting site to scrape.")
PARSER.add_argument('-l', '--limit', type=int, default=0,
                    help="Optional maximum number of posts to request from "
                         "the text-pasting site.")
ARGS = PARSER.parse_args()

doxscrapers.reporting.configure_logger(None, True, ARGS.site)

SCRAPER = SITES[ARGS.site](None)
ITEMS = [i.to_dict() for i in SCRAPER.posts(limit=ARGS.limit)]

print json.dumps(ITEMS)
