#!/usr/bin/env python

import argparse
import doxextractor.reporting
import doxextractor.controllers
import tornado.web

parser = argparse.ArgumentParser(description="Web service for extracting "
                                             "information out of possible"
                                             "dox files.")
parser.add_argument('-p', '--port', default=8888, type=int,
                    help="The port the webserver should listen on.")
parser.add_argument('-i', '--ip', default="0.0.0.0",
                    help="The IP address to listen on.")
parser.add_argument('-l', '--log', default=None,
                    help="Path to write logs to.")
parser.add_argument('-v', '--verbose', action="store_true",
                    help="Enables debugging / verbose logging.")
ARGS = parser.parse_args()

doxextractor.reporting.configure_logger(ARGS.log, ARGS.verbose)

SETTINGS = {
    "debug": ARGS.verbose,
    "log_function": doxextractor.reporting.http_error_logger,
}

APPLICATION = tornado.web.Application([
    (r'/extract', doxextractor.controllers.ExtractionHandler)
], **SETTINGS)
APPLICATION.listen(ARGS.port, address=ARGS.ip)
tornado.ioloop.IOLoop.current().start()
