#!/usr/bin/env python

import sys
import codecs
import fileinput
import collections
from tabulate import tabulate
import doxextractor.tags as et
import doxextractor.personal as ep
import doxextractor.online as eo

tags_to_measure = (
    ('NAME:first', ep.first_name),
    ('NAME:last', ep.last_name),
    ('AGE', ep.age),
    ('PHONE', ep.phone),
    ('ONLINE:skype', eo.skype),
    ('ONLINE:ig', eo.instagram),
    ('ONLINE:twitter', eo.twitter),
    ('ONLINE:kik', eo.kik),
    ('ONLINE:yt', eo.youtube),
    ('ONLINE:gplus', eo.google_plus),
    ('ONLINE:fb', eo.facebook),
    ('ONLINE:twitch', eo.twitch),
)

# tagname -> (num truth observations, num predicted, correct_predictions)
measurements = collections.OrderedDict([(tagname, [0, 0, 0]) for tagname, func in tags_to_measure])
def add_datapoint_to_measurements(tagname, text, truth, func, filename=None):
    found_tokens = func(text)

    try:
        true_tokens = truth[tagname]
    except KeyError:
        true_tokens = []

    first_true_token = None if len(true_tokens) == 0 else true_tokens[0]
    first_predicted_token = None if len(found_tokens) == 0 else found_tokens[0]

    if first_true_token is not None:
        measurements[tagname][0] += 1

    if first_predicted_token is not None:
        measurements[tagname][1] += 1

    if first_true_token == first_predicted_token:
        measurements[tagname][2] += 1
    elif filename:
        print "ERROR: {}, {} (expected {}, got {})".format(filename, tagname, first_true_token, first_predicted_token)


num_files = 0.0
for filename in fileinput.input():
    with codecs.open(filename.strip(), 'r') as h:
        num_files += 1.0
        all_text = "".join(h)
        stripped_text = et.strip(all_text)
        truth_tokens = et.extract(all_text)

        for tagname, func in tags_to_measure:
            add_datapoint_to_measurements(tagname, stripped_text, truth_tokens, func, filename=filename.strip())

rows = []
for tagname, data in measurements.items():
    data_pct = [d / num_files for d in data]
    data_pct.insert(0, tagname)
    rows.append(data_pct)

print tabulate(rows, headers=["Tags", "% using", "% with Guess", "% Correct"])
